@@ -2,9 +2,7 @@
 #include <Rcpp.h>
 #include <map>
 #include <queue>
-#include <boost/shared_ptr.hpp>
-#include <boost/make_shared.hpp>
-#include <boost/scope_exit.hpp>
+#include <memory>
 #include "debug.h"
 #include "utils.h"
 #include "threadutils.h"
@@ -14,11 +12,12 @@
 
 #include "interrupt.h"
 
-using boost::shared_ptr;
+using std::shared_ptr;
 
 static size_t exec_callbacks_reentrancy_count = 0;
 
-static CallbackRegistryTable callbackRegistryTable;
+// instance has global scope as declared in callback_registry_table.h
+CallbackRegistryTable callbackRegistryTable;
 
 
 class ProtectCallbacks {
@@ -206,14 +205,21 @@ bool execCallbacksOne(
     if (callbacks.size() == 0) {
       break;
     }
+
+#ifdef RCPP_USING_UNWIND_PROTECT // See https://github.com/r-lib/later/issues/191
+    // This line may throw errors!
+    callbacks[0]->invoke();
+#else
     // This line may throw errors!
     callbacks[0]->invoke_wrapped();
+#endif
+
   } while (runAll);
 
   // I think there's no need to lock this since it's only modified from the
   // main thread. But need to check.
-  std::vector<boost::shared_ptr<CallbackRegistry> > children = callback_registry->children;
-  for (std::vector<boost::shared_ptr<CallbackRegistry> >::iterator it = children.begin();
+  std::vector<std::shared_ptr<CallbackRegistry> > children = callback_registry->children;
+  for (std::vector<std::shared_ptr<CallbackRegistry> >::iterator it = children.begin();
        it != children.end();
        ++it)
   {
